const St = imports.gi.St;
const Atk = imports.gi.Atk;
const Main = imports.ui.main;
const Search = imports.ui.search;
const SearchDisplay = imports.ui.searchDisplay;
const IconGrid = imports.ui.iconGrid;
const Gio = imports.gi.Gio;
const Soup = imports.gi.Soup;
const Util = imports.misc.util;
const URLHighlighter = imports.ui.messageTray.URLHighlighter;

const _httpSession = new Soup.SessionSync();
Soup.Session.prototype.add_feature.call(
    _httpSession,
    new Soup.ProxyResolverDefault()
);
_httpSession.user_agent = 'Gnome-Shell Wikipedia Search Provider';

const MAX_SEARCH_RESULTS_ROWS = 5;
const MAX_SEARCH_RESULTS_COLUMNS = 2
const ICON_SIZE = 120;


const WIKIPEDIA_KEYWORD = /(w|w-(.*?)):(.*)/;
const WIKIPEDIA_DOMAIN = "wikipedia.org";
const WIKIPEDIA_API_URL = "/w/api.php";
const WIKIPEDIA_DEFAULT_LANGUAGE = "en";

let wikipediaProvider = "";

function WikipediaResultActor(result) {
    this._init(result);
}

WikipediaResultActor.prototype = {
    _init: function(resultMeta) {
        this.actor = new St.Bin({
            style_class: 'wikipedia',
            reactive: true,
            can_focus: true,
            track_hover: true,
            accessible_role: Atk.Role.PUSH_BUTTON
        });

        let content = new St.BoxLayout({
            style_class: 'wikipedia-content',
            vertical: false
        });
        this.actor.set_child(content);

        if(resultMeta.show_icon){
            let icon = new St.Icon({
                icon_type: St.IconType.FULLCOLOR,
                icon_size: ICON_SIZE,
                icon_name: 'wikipedia',
                style_class: 'wikipedia-icon'
            });

            content.add(icon, {
                x_fill: true,
                y_fill: false,
                x_align: St.Align.START,
                y_align: St.Align.MIDDLE
            });
        }

        let details = new St.BoxLayout({
            style_class: 'wikipedia-details',
            vertical: true
        });

        content.add(details, {
            x_fill: true,
            y_fill: false,
            x_align: St.Align.START,
            y_align: St.Align.MIDDLE
        });

        let title = new St.Label({
            text: resultMeta.title,
            style_class: 'wikipedia-details-title'
        });

        details.add(title, {
            x_fill: true,
            y_fill: false,
            x_align: St.Align.START,
            y_align: St.Align.START
        });
        this.actor.label_actor = title;

        let extract_box = new St.BoxLayout({
            vertical: true,
            style_class: 'wikipedia-details-extract'
        });

        let extract = new URLHighlighter(
            resultMeta.extract,
            true,
            false
        );

        extract_box.add(extract.actor, {
            x_fill: true,
            y_fill: false,
            x_align: St.Align.END,
            y_align: St.Align.START
        });
        details.add(extract_box, {
            x_fill: false,
            y_fill: true,
            x_align: St.Align.START,
            y_align: St.Align.END
        });
    }
};

function WikipediaProvider() {
    this._init.apply(this, arguments);
}

WikipediaProvider.prototype = {
    __proto__: Search.SearchProvider.prototype,

    _init: function(title) {
        Search.SearchProvider.prototype._init.call(this, title);
    },

    _parse_query: function(terms_string) {
        let result = {};
        result.lang = '';
        result.go = false;
        result.wikipedia_query = false;

        if(WIKIPEDIA_KEYWORD.test(terms_string)) {
            result.wikipedia_query = true;
            let matches = WIKIPEDIA_KEYWORD.exec(terms_string);

            if(matches[2]) {
                result.lang = matches[2];
            }
            if(matches[3]) {
                result.term = matches[3];

                if(result.term.slice(-1) == ":") {
                    result.go = true;
                    result.term = result.term.slice(0, -1);
                }
            }
        }

        return result;
    },

    _get_wikipedia_url: function (language, api_url, api_query_string) {
        let result_url = '';
        let protocol = "http://";

        result_url = protocol+language+'.'+WIKIPEDIA_DOMAIN;

        if(api_url){
            result_url += api_url;
        }
        if(api_query_string){
            result_url += '?'+api_query_string;
        }

        return result_url;
    },

    _get_wiki_extracts: function(titles, language) {
        let titles_string = '';

        for (let i = 0; i < titles.length; i++){
            titles_string = titles_string+titles[i];
            
            if(i != (titles.length - 1)){
                titles_string += '|';
            }
        }

        if(titles_string){
            titles_string = encodeURIComponent(titles_string);
            let exlimit = MAX_SEARCH_RESULTS_ROWS * MAX_SEARCH_RESULTS_COLUMNS;
            let api_query_extracts = "action=query&prop=extracts&format=json&exlimit="+exlimit+"&explaintext&exsectionformat=plain&exchars=300&exintro=&redirects=&titles="+titles_string;
            let url = this._get_wikipedia_url(language, WIKIPEDIA_API_URL, api_query_extracts);
            let msg = Soup.Message.new("GET", url);
            _httpSession.send_message(msg);

            if(msg.status_code === 200) {
                let result = JSON.parse(msg.response_body.data);
                return result['query']['pages'];
            }
            else{
                log('response code != 200');
                return false;
            }
        }
        else{
            log('empty titles_string');
            return false;
        }
    },

    _get_wiki_titles: function(term, language) {
        if(term){
            term = encodeURIComponent(term);
            let exlimit = MAX_SEARCH_RESULTS_ROWS * MAX_SEARCH_RESULTS_COLUMNS;
            let api_query_titles = "action=opensearch&format=json&limit="+exlimit+"&search="+term;
            let url = this._get_wikipedia_url(language, WIKIPEDIA_API_URL, api_query_titles);
            let msg = Soup.Message.new("GET", url);
            _httpSession.send_message(msg);

            if(msg.status_code === 200) {
                let result = JSON.parse(msg.response_body.data);
                return result[1];
            }
            else{
                log('response_code != 200');
                return false;
            }
        }
        else{
            log('empty term');
            return false;
        }
    },

    getInitialResultSet: function(terms) {
        let language = WIKIPEDIA_DEFAULT_LANGUAGE;
        let terms_string = terms.join(" ");
        let query = this._parse_query(terms_string);

        if(query.wikipedia_query) {
            if(query.go) {
                if(query.lang.length > 0) {
                    language = query.lang;
                }

                let titles = this._get_wiki_titles(query.term, language);
                let result = [];

                if(titles){
                    let extracts = this._get_wiki_extracts(titles, language);

                    if(extracts) {
                        for(let id in extracts) {
                            if(extracts[id]['title'] && extracts[id]['extract']) {
                                result.push({
                                    "id": id,
                                    "title": extracts[id]['title'],
                                    "extract": extracts[id]['extract']
                                });
                            }
                            else {
                                // Main.notify(JSON.stringify(extracts[id]));
                            }
                        }
                    }
                }

                if(result.length > 0) {
                    return result;
                }
                else {
                    let nothing_found = [{
                        "title": "Wikipedia Search Provider",
                        "extract": "Your search - "+query.term+" - did not match any documents.",
                        "show_icon": true
                    }]
                    return nothing_found;
                }
            }
            else {
                let wellcome = [{
                    "title": "Wikipedia Search Provider",
                    "extract": "Enter your query, don't forget put ':' at the end.",
                    "show_icon": true
                }]
                return wellcome;
            }
        }

        return [];
    },

    getSubsearchResultSet: function(prevResults, terms) {
        return this.getInitialResultSet(terms);
    },

    getResultMetas: function(result) {
        let metas = [];
        for(let i = 0; i < result.length; i++) {
            metas.push({
                'id' : result[i].id,
                'extract' : result[i].extract,
                'title' : result[i].title,
                'show_icon': result[i].show_icon
            });
        }
        return metas;
    },

    createResultActor: function(resultMeta, terms) {
        let result = new WikipediaResultActor(resultMeta);
        return result.actor;
    },

    createResultContainerActor: function() {
        let grid = new IconGrid.IconGrid({
            rowLimit: MAX_SEARCH_RESULTS_ROWS,
            //columnLimit: MAX_SEARCH_RESULTS_COLUMNS,
            xAlign: St.Align.START
        });
        grid.actor.style_class = 'wikipedia-grid';

        let actor = new SearchDisplay.GridSearchResults(this, grid);
        return actor;
    },

    activateResult: function(resultId) {
        resultId = parseInt(resultId);
        let url = false;

        if(resultId) {
            url = WIKIPEDIA_URL+'?curid='+resultId;
        }

        if(url) {
            try {
                Gio.app_info_launch_default_for_uri(
                    url,
                    global.create_app_launch_context()
                );
            }
            catch (e) {
                Util.spawn(['gvfs-open', url])
            }
        }
        else {
            // Main.notify("Bad url.");
        }

        return true;
    }
}

function init() {
    wikipediaProvider = new WikipediaProvider('WIKIPEDIA');
}

function enable() {
    Main.overview.addSearchProvider(wikipediaProvider);
}

function disable() {
    Main.overview.removeSearchProvider(wikipediaProvider);
}
